/**********************************************************************
* @file		A31G31x_comp.h
* @brief	Contains all macro definitions and function prototypes
* 			support for UART firmware library on A31G31x
* @version	1.0
* @date		
* @author ABOV M team
*
* Copyright(C)  2015, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/

#ifndef _COMP_H_
#define _COMP_H_

/* Includes ------------------------------------------------------------------- */
#include "A31G31x.h"
//#include "def.h"
#include "aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif


/* Private Macros ------------------------------------------------------------- */

/* --------------------- BIT DEFINITIONS -------------------------------------- */
/**********************************************************************
 * Macro defines for Macro defines for Comparator control register
 **********************************************************************/
#define CMP_CR_INPSEL_CP0    		((uint32_t)(0))   		
#define CMP_CR_INPSEL_CP1A   		((uint32_t)(0))   		
#define CMP_CR_INPSEL_CP1B   		((uint32_t)(1<<0))   		
#define CMP_CR_INPSEL_CP1C  		((uint32_t)(2<<0))   		
#define CMP_CR_INPSEL_BITMASK 		((uint32_t)(3<<0))   		
	
#define CMP_CR_INNSEL_CREF    		((uint32_t)(0<<4))   	
#define CMP_CR_INNSEL_BRG1V    		((uint32_t)(1<<4))   	
#define CMP_CR_INNSEL_DACOUT    		((uint32_t)(2<<4))   	
#define CMP_CR_INNSEL_BITMASK    		((uint32_t)(3<<4))   	
	
#define CMP_CR_DISEN    		((uint32_t)(0<<12))   	
#define CMP_CR_EN     		((uint32_t)(1<<12))   	
	
#define CMP_CR_HYSSEL_5MV     		((uint32_t)(0<<16))   	
#define CMP_CR_HYSSEL_20MV     		((uint32_t)(1<<16))   	
#define CMP_CR_HYSSEL_BITMASK     		((uint32_t)(1<<16))   	

#define CMP_CR_HYSDISEN	((uint32_t)(0<<20))   	
#define CMP_CR_HYSEN	((uint32_t)(1<<20))   	

#define CMP_ICON_BITMASK		((uint32_t)(0x330F))		

/**********************************************************************
 * Macro defines for Macro defines for Comparator debounce register
 **********************************************************************/
#define CMP_DBNC_C0DBNC	((uint32_t)(0))	
#define CMP_DBNC_C0DBNC_BITMASK	((uint32_t)(0xFuL<<0))	

#define CMP_DBNC_C1DBNC		((uint32_t)(0<<4)) 	
#define CMP_DBNC_C1DBNC_BITMASK		((uint32_t)(0xFuL<<4)) 

#define CMP_DBNC_DBNCTB		((uint32_t)(0<<16)) 	
#define CMP_DBNC_DBNCTB_BITMASK		((uint32_t)(0xFFFFuL<<16)) 	

#define CMP_DBNC_BITMASK		((uint32_t)(0xFFFF00FF))	

/**********************************************************************
 * Macro defines for Macro defines for Comparator Interrupt control register
 **********************************************************************/
#define CMP_ICON_C0IMODE   		((uint32_t)(0<<0))   		
#define CMP_ICON_C0IMODE_BITMASK  		((uint32_t)(3<<0))   		

#define CMP_ICON_C1IMODE     		((uint32_t)(0<<2))   	
#define CMP_ICON_C1IMODE_BITMASK     		((uint32_t)(3<<2))   	

#define CMP_ICON_IPOL0     		((uint32_t)(0<<8))   	
#define CMP_ICON_IPOL0_BITMASK    		((uint32_t)(1<<8))   	

#define CMP_ICON_IPOL1    		((uint32_t)(0<<9))   	
#define CMP_ICON_IPOL1_BITMASK    		((uint32_t)(1<<9))   	

#define CMP_ICON_TPOL0	((uint32_t)(0<<12))   	
#define CMP_ICON_TPOL0_BITMASK	((uint32_t)(1<<12))   

#define CMP_ICON_TPOL1		((uint32_t)(0<<13))		
#define CMP_ICON_TPOL1_BITMASK		((uint32_t)(1<<13))		

#define CMP_ICON_BITMASK		((uint32_t)(0x330F))		

/**********************************************************************
 * Macro defines for Macro defines for Comparator interrupt enable register
 **********************************************************************/
#define CMP_IEN_C0_EN		((uint8_t)(1<<0)) 	
#define CMP_IEN_C1_EN		((uint8_t)(1<<1)) 	
#define CMP_IEN_BITMASK		((uint8_t)(0x03)) 

/**********************************************************************
 * Macro defines for Macro defines for Comparator Interrupt status register
 **********************************************************************/
#define CMP_IST_C0IRQ_ON		((uint8_t)(1<<0)) 	
#define CMP_IST_C1IRQ_ON			((uint8_t)(1<<1)) 	
#define CMP_IST_BITMASK	((uint8_t)(0x03)) 

/**********************************************************************
 * Macro defines for Macro defines for Comparator Interrupt clear register
 **********************************************************************/
#define CMP_ICLR_C0ICLR		((uint8_t)(1<<0)) 	
#define CMP_ICLR_C1ICLR		((uint8_t)(1<<1)) 	
#define CMP_ICLR_BITMASK		((uint8_t)(0x03)) 	


/* Public Types --------------------------------------------------------------- */
/***********************************************************************
 * @brief Comparator enumeration
**********************************************************************/


 /* Comparator interrupt Flag definition */
enum {
	ICON_LEVEL		= 0,     		
	ICON_RISING_EDGE_INT,		     			
	ICON_FALLING_EDGE_INT,		     		
	ICON_BOTH_EDGE_INT		     			
};
 /* Comparator interrupt polarity(level mode) definition */
enum {
	CMP_INTOUT_HIGH		= 0,   					
	CMP_INTOUT_LOW		 							
};
 /* Comparator trigger output polarity(to trigger other IP) definition */
enum {
	CMP_TRIOUT_NORMAL		= 0,   					
	CMP_TRIOUT_INVERTED		 							
};

 /* Comparator Hysteresis select definition */
enum {
	CMP_HYSSEL_5MV	= 0,   					
	CMP_HYSSEL_20MV		 							
};
 /* Comparator input(input +) select definition */
enum {
	CMP_INPUT_CP1A	= 0,   					
	CMP_INPUT_CP0 = CMP_INPUT_CP1A,
	CMP_INPUT_CP1B,	
	CMP_INPUT_CP1C
};
 /* Comparator reference(input -) select definition */
enum {
	CMP_REF_CREF	= 0,   					
	CMP_REF_BGR1V,		 							
	CMP_REF_DACOUT
};
 /* Comparator Select definition */
enum {
	CMP0 = 0,
	CMP1
};


/* Public Functions ----------------------------------------------------------- */
void CMP_Init(void);
/* Comparator interrupt status get/clear functions --------------------------------------------------*/
uint32_t CMP_GetIntStatusSelect(uint32_t comp_no);
uint32_t CMP_GetIntStatusAll(void);
void CMP_ClearIntSelect(uint32_t comp_no);
void CMP_ClearIntAll(void);

/* Comparator configure functions -------------------------------------------------*/
void CMP_HysterisisConfigure(uint32_t comp_no, uint32_t hyssel, FunctionalState NewState);
void CMP_ConfigureFunction(uint32_t comp_no, uint32_t refsel, uint32_t inputsel);
void CMP_Debounce(uint32_t comp_no, uint32_t dbncshift, uint32_t dbnccounter);
void CMP_IntConfigure(uint32_t comp_no, uint32_t tpol, uint32_t ipol, uint32_t mode);

/* Comparator operate functions -------------------------------------------------*/
void CMP_IntCmd(uint32_t comp_no, FunctionalState NewState);
void CMP_Cmd(uint32_t comp_no,  FunctionalState NewState);

#ifdef __cplusplus
}
#endif


#endif /* _COMP_H_ */

/* --------------------------------- End Of File ------------------------------ */
