/**********************************************************************
* @file		A31G31x_lcd.h
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library on A31G31x
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2015, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/

#ifndef _A31G31x_LCD_H_
#define _A31G31x_LCD_H_

/* Includes ------------------------------------------------------------------- */
#include "A31G31x.h"
#include "aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/* Private macros ------------------------------------------------------------- */

/* ================================================================================ */
/* ================          LCD	Control Register				================ */
/* ================================================================================ */
#define LCDCR_INTBIAS						(0x0uL)
#define LCDCR_EXTBIAS						(0x1uL)

#define LCDCR_RLCD3							(0x0uL)
#define LCDCR_RLCD1							(0x1uL)
#define LCDCR_RLCD2							(0x2uL)
#define LCDCR_RLCD4							(0x3uL)

#define LCDCR_Duty_div8_Bias_div4			(0x0uL)
#define LCDCR_Duty_div6_Bias_div4			(0x1uL)
#define LCDCR_Duty_div5_Bias_div3			(0x2uL)
#define LCDCR_Duty_div4_Bias_div3			(0x3uL)
#define LCDCR_Duty_div3_Bias_div3			(0x4uL)
#define LCDCR_Duty_div3_Bias_div2			(0x5uL)

#define LCDCR_LCDCLK_128Hz					(0x0uL)
#define LCDCR_LCDCLK_256Hz					(0x1uL)
#define LCDCR_LCDCLK_512Hz					(0x2uL)
#define LCDCR_LCDCLK_1024Hz					(0x3uL)

#define LCDCR_DISP_OFF						(0x0uL)
#define LCDCR_DISP_ON						(0x1uL)


/* ================================================================================ */
/* ================          LCD	Bias & Contrast Register				================ */
/* ================================================================================ */
#define LCDBCCR_AUTOBIAS_OFF				(0x0uL)
#define LCDBCCR_AUTOBIAS_ON					(0x1uL)

#define LCDBCCR_BMSEL_1CLK					(0x0uL)
#define LCDBCCR_BMSEL_2CLK					(0x1uL)
#define LCDBCCR_BMSEL_3CLK					(0x2uL)
#define LCDBCCR_BMSEL_4CLK					(0x3uL)
#define LCDBCCR_BMSEL_5CLK					(0x4uL)
#define LCDBCCR_BMSEL_6CLK					(0x5uL)
#define LCDBCCR_BMSEL_7CLK					(0x6uL)
#define LCDBCCR_BMSEL_8CLK					(0x7uL)

#define LCDBCCR_VLCD_31STEP					(0x0uL)
#define LCDBCCR_VLCD_30STEP					(0x1uL)
#define LCDBCCR_VLCD_29STEP					(0x2uL)
#define LCDBCCR_VLCD_28STEP					(0x3uL)
#define LCDBCCR_VLCD_27STEP					(0x4uL)
#define LCDBCCR_VLCD_26STEP					(0x5uL)
#define LCDBCCR_VLCD_25STEP					(0x6uL)
#define LCDBCCR_VLCD_24STEP					(0x7uL)
#define LCDBCCR_VLCD_23STEP					(0x8uL)
#define LCDBCCR_VLCD_22STEP					(0x9uL)
#define LCDBCCR_VLCD_21STEP					(0xAuL)
#define LCDBCCR_VLCD_20STEP					(0xBuL)
#define LCDBCCR_VLCD_19STEP					(0xCuL)
#define LCDBCCR_VLCD_18STEP					(0xDuL)
#define LCDBCCR_VLCD_17STEP					(0xEuL)
#define LCDBCCR_VLCD_16STEP					(0xFuL)





/*
 * @brief 	PCU port mode enumerate definition
 */
typedef struct {
	uint32_t	LCDDRSel;
	uint32_t	IrSel;
	uint32_t	DutyBias;
	uint32_t	LCDCLK;
	uint32_t	DISPSel;
	
	uint32_t	LCDAutoBias;
	uint32_t	BiasModeSel;
	uint32_t	LCTSel;
	uint32_t	VLCDVal;
	
	uint32_t	externBiasEn;
}LCD_CFG_Type;



/* Public Functions ----------------------------------------------------------- */
void LCD_Init(LCD_Type *LCDx, LCD_CFG_Type *LCD_Config);
void LCD_DISPEN(LCD_Type *LCDx);
void LCD_DISPDIS(LCD_Type *LCDx);




#ifdef __cplusplus
}
#endif

#endif /* end _A31G31x_lcd_H_ */

/* --------------------------------- End Of File ------------------------------ */
