/**********************************************************************
* @file		A31G216X_adc.h
* @brief	Contains all macro definitions and function prototypes
* 			support for ADC firmware library on A31G216X
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2015, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/

#ifndef _A31G216X_ADC_H_
#define _A31G216X_ADC_H_

/* Includes ------------------------------------------------------------------- */
#include "A31G216X.h"
#include "aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif

/* Private macros ------------------------------------------------------------- */

/* -------------------------- BIT DEFINITIONS ----------------------------------- */
/**********************************************************************
 * Macro defines for ADC  control register
 **********************************************************************/
/**  Start mask bits */
#define ADC_CR_START_MASK	((1UL<<0))
/**  Stop mask bits */
#define ADC_CR_STOP_MASK	((1UL<<7))

/**********************************************************************
 * Macro defines for ADC Status register
**********************************************************************/
#define ADC_STAT_END					((1UL<<4))

/**********************************************************************
 * Macro defines for ADC Interrupt register
 **********************************************************************/
/** These bits allow control over which A/D channels generate
 * interrupts for conversion completion */
#define ADC_INTEN					((1UL<<5))

/**********************************************************************
 * Macro defines for ADC Data register
 **********************************************************************/
/** When DONE is 1, this field contains result value of ADC conversion */
#define ADC_DR_RESULT(n)			((n)&0xFFF)


/* Public Types --------------------------------------------------------------- */
#define ADC_REF_VDD                  0  /*!< ADC ref source VDD */
#define ADC_REF_AVREF               1  /*!< ADC ref source AVREF */

#define ADC_TRIGGER_DISABLE           0  /*!< Event Trigger Disabled/Soft-Trigger Only */
#define ADC_TRIGGER_TIMER10           1  /*!< Timer10 Event Trigger */
#define ADC_TRIGGER_TIMER11           2  /*!< Timer11 Event Trigger */
#define ADC_TRIGGER_TIMER12           3  /*!< Timer12 Event Trigger */
#define ADC_TRIGGER_TIMER30           4  /*!< Timer30 Event Trigger */

typedef struct {
/* ADCCR */	
	uint32_t DmaOpt;					/**< DmaOpt = ENABLE or DISABLE */
	uint32_t RefSel;				/**< RestartEn = ENABLE or DISABLE */			
	uint32_t TrgSel;	
	
/* ADPREDR */
	uint32_t InClkDiv;
} ADC_CFG_Type;



/* Public Functions ----------------------------------------------------------- */
/* Init/DeInit ADC peripheral ----------------*/
void ADC_Init(ADC_Type *ADCx, ADC_CFG_Type *ADC_ConfigStruct);
void ADC_DeInit(ADC_Type *ADCx);

/* Enable/Disable ADC functions --------------*/
void ADC_PowerdownCmd(ADC_Type *ADCx, FunctionalState NewState);

void ADC_ChannelSel(ADC_Type *ADCx, uint32_t Channel);

void ADC_StartCmd(ADC_Type *ADCx);

/* Configure ADC functions -------------------*/
void ADC_EdgeStartConfig(ADC_Type *ADCx, uint8_t EdgeOption);
void ADC_IntConfig(ADC_Type *ADCx, FunctionalState NewState);

/* Get ADC information functions -------------------*/
uint16_t ADC_GetData(ADC_Type *ADCx);

/* Get/Clear ADC status  -------------------*/
uint32_t ADC_GetStatus(ADC_Type *ADCx);
void ADC_ClearStatus(ADC_Type *ADCx);

#ifdef __cplusplus
}
#endif


#endif /* _A31G216X_ADC_H_ */

/* --------------------------------- End Of File ------------------------------ */
