/**********************************************************************
* @file		A31G31x_lcd.c
* @brief	Contains all functions support for GPIO firmware library
* 			on A31G31x
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2015, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "A31G31x_scu.h"
#include "A31G31x_lcd.h"


/* LCD ------------------------------------------------------------------------------ */

/**********************************************************************
 * @brief		LCD Init
 * @param[in]	LCD Type, LCD Config Struct
 * @return		None
 **********************************************************************/
void LCD_Init(LCD_Type *LCDx, LCD_CFG_Type *LCD_Config)
{
	// LCD Block Enable
	SCU->PER2 &= ~(1<<28);
	SCU->PCER2 &= ~(1<<28);
	
	SCU->PER2 |= (1<<28);
	SCU->PCER2 |= (1<<28);
	
	LCD->CR = 0
	| ((LCD_Config->LCDDRSel&(1))<<8)
	| ((LCD_Config->IrSel&(3))<<6)
	| ((LCD_Config->DutyBias&(7))<<3)
	| ((LCD_Config->LCDCLK&(3))<<1)
	| ((LCD_Config->DISPSel&(1))<<0)
	;
	
	LCD->BCCR = 0
	| ((LCD_Config->LCDAutoBias&(1))<<12)
	| ((LCD_Config->BiasModeSel&(7))<<8)
	| ((LCD_Config->LCTSel&(1))<<5)
	| ((LCD_Config->VLCDVal&(0xF))<<0)
	;

	if(LCD_Config->externBiasEn == ENABLE){
		LCD->BSSR = 0xBF0;
	}else{
		LCD->BSSR = 0;
	}
}

//void LCD_



/**********************************************************************
 * @brief		LCD Display On
 * @param[in]	LCD Type
 * @return		None
 **********************************************************************/
void LCD_DISPEN(LCD_Type *LCDx)
{
	LCDx->CR |= (1<<0);
}


/**********************************************************************
 * @brief		LCD Display Off
 * @param[in]	
 * @return		None
 **********************************************************************/
void LCD_DISPDIS(LCD_Type *LCDx)
{
	LCDx->CR &= ~(1<<0);
}


/* --------------------------------- End Of File ------------------------------ */
