#include "A31G31x_pcu.h"
#include "A31G31x_scu.h"
#include "A31G31x_uartn.h"
#include "debug_frmwrk.h"
#include "slib.h"

#define 	cli() __disable_irq()
#define 	sei() __enable_irq()

extern uint32_t msec;
unsigned char ring_state;

uint8_t *lcdfbuff;	// com0~com3,seg7~seg15

const char LEDSEG[17]={
	//0    1    2    3    4    5    6    7    8    9    A    b    c    d    E    F   ' '
	0x0f,0x00,0x0b,0x09,0x04,0x0d,0x0f,0x0c,0x0f,0x0d,0x0e,0x07,0x0f,0x03,0x0f,0x0e,0x00	
};
const char LEDSEGn[17]={
	//0    1    2    3    4    5    6    7    8    9    A    b    c    d    E    F   ' '
	0x0a,0x0a,0x0c,0x0e,0x0e,0x06,0x06,0x0a,0x0e,0x0e,0x0e,0x06,0x00,0x0e,0x0e,0x0e,0x00	
};

void l_init(void)
{
	int i;

	for (i=0;i<27;i++)
		lcdfbuff[i]=0x00;
}

void l_disp(unsigned int disp)
{
	char tch[6];
	
	tch[0]=disp%10; disp/=10;
	if (disp==0){
		tch[1]=tch[2]=tch[3]=16;
	}
	else {
		tch[1]=disp%10; disp/=10;
		if (disp==0){
			tch[2]=tch[3]=16;
		}
		else {
			tch[2]=disp%10; disp/=10;
			if (disp==0){
				tch[3]=16;
			}
			else {
				tch[3]=disp%10;
			}
		}
	}

	lcdfbuff[7]=LEDSEG[tch[3]];
	lcdfbuff[8]=LEDSEGn[tch[3]];
	lcdfbuff[9]=LEDSEG[tch[2]];
	lcdfbuff[10]=LEDSEGn[tch[2]];
	lcdfbuff[11]=LEDSEG[tch[1]];
	lcdfbuff[12]=LEDSEGn[tch[1]];
	lcdfbuff[13]=LEDSEG[tch[0]];
	lcdfbuff[14]=LEDSEGn[tch[0]];
}


void l_ring(char dir)
{
	char tch[8];

	if (dir){
		ring_state++;
		if (ring_state>11) ring_state=0;
	}
	else {
		ring_state--;
		if (ring_state==0xff) ring_state=11;
	}
	
	switch(ring_state){
		case  0:tch[0]=0x08;tch[1]=0   ;tch[2]=0x08;tch[3]=0   ;tch[4]=0   ;tch[5]=0   ;tch[6]=0   ;tch[7]=0   ;break;
		case  1:tch[0]=0   ;tch[1]=0   ;tch[2]=0x08;tch[3]=0   ;tch[4]=0x08;tch[5]=0   ;tch[6]=0   ;tch[7]=0   ;break;
		case  2:tch[0]=0   ;tch[1]=0   ;tch[2]=0   ;tch[3]=0   ;tch[4]=0x08;tch[5]=0   ;tch[6]=0x08;tch[7]=0   ;break;
		case  3:tch[0]=0   ;tch[1]=0   ;tch[2]=0   ;tch[3]=0   ;tch[4]=0   ;tch[5]=0   ;tch[6]=0x08;tch[7]=0x08;break;
		case  4:tch[0]=0   ;tch[1]=0   ;tch[2]=0   ;tch[3]=0   ;tch[4]=0   ;tch[5]=0   ;tch[6]=0   ;tch[7]=0x0a;break;
		case  5:tch[0]=0   ;tch[1]=0   ;tch[2]=0   ;tch[3]=0   ;tch[4]=0   ;tch[5]=0   ;tch[6]=0x01;tch[7]=0x02;break;
		case  6:tch[0]=0   ;tch[1]=0   ;tch[2]=0   ;tch[3]=0   ;tch[4]=0x01;tch[5]=0   ;tch[6]=0x01;tch[7]=0   ;break;
		case  7:tch[0]=0   ;tch[1]=0   ;tch[2]=0x01;tch[3]=0   ;tch[4]=0x01;tch[5]=0   ;tch[6]=0   ;tch[7]=0   ;break;
		case  8:tch[0]=0x01;tch[1]=0   ;tch[2]=0x01;tch[3]=0   ;tch[4]=0   ;tch[5]=0   ;tch[6]=0   ;tch[7]=0   ;break;
		case  9:tch[0]=0x03;tch[1]=0   ;tch[2]=0   ;tch[3]=0   ;tch[4]=0   ;tch[5]=0   ;tch[6]=0   ;tch[7]=0   ;break;
		case 10:tch[0]=0x06;tch[1]=0   ;tch[2]=0   ;tch[3]=0   ;tch[4]=0   ;tch[5]=0   ;tch[6]=0   ;tch[7]=0   ;break;
		case 11:tch[0]=0x0c;tch[1]=0   ;tch[2]=0   ;tch[3]=0   ;tch[4]=0   ;tch[5]=0   ;tch[6]=0   ;tch[7]=0   ;break;
		default: break;
	}

	cli();
	lcdfbuff[7]=tch[0];
	lcdfbuff[8]=tch[1];	
	lcdfbuff[9]=tch[2];
	lcdfbuff[10]=tch[3];	
	lcdfbuff[11]=tch[4];
 	lcdfbuff[12]=tch[5];
	lcdfbuff[13]=tch[6];
	lcdfbuff[14]=tch[7];
	lcdfbuff[15]=0;	
	sei();			   
}

void l_bar(void)
{
	cli();
	lcdfbuff[7]=0;
	lcdfbuff[8]=0x04;	
	lcdfbuff[9]=0;
	lcdfbuff[10]=0x04;	
	lcdfbuff[11]=0;
 	lcdfbuff[12]=0x04;
	lcdfbuff[13]=0;
	lcdfbuff[14]=0x04;
	lcdfbuff[15]=0;
	sei();			   
}

void LCD_TEST(void)
{
	char			cmd;
	unsigned int	count=0;
	
	lcdfbuff = (uint8_t *)(0x40005010);

	cli();l_init();sei();

	_DBG("\n\rLCD test Program\n\r");
	_DBG("1. LCD Number counting test\n\r");
	_DBG("2. LCD CCW Spin patten test\n\r");
	_DBG("3. LCD CW Spin patten test\n\r");
	_DBG("4. LCD Center Line\n\r");
	
	_DBG(">");

	_DBG("\n\r");

	cmd = _DG;		
	while(1)
	{
		switch (cmd)
		{
			case '1':
				l_disp(count++);
				msec=200;
				while(msec);
				break;
			case '2':
				l_ring(0);
				msec=40;
				while(msec);
				count=0;
				break;
			case '3':
				l_ring(1);
				msec=60;
				while(msec);
				count=0;
				break;
			case '4':
				l_bar();
				msec=60;
				while(msec);
				count=0;
				break;
			default :
				return;
			
		}
		
	}
}


