#include "A31G31x_pcu.h"
#include "A31G31x_scu.h"
#include "usb.h"


uint8_t device_desc[] = {
	0x12,                       /* bLength */
	eUSB_DT_DEVICE,             /* bDescriptorType */
	LSB(USB_11_SPEC_VER),       /* bcdUSB */
	MSB(USB_11_SPEC_VER),
	0x00,                       /* bDeviceClass */
	0x00,                       /* bDeviceSubClass */
	0x00,                       /* bDeviceProtocol */
	USB_EP0_MAX_PKT_SZ,         /* bMaxPacketSize */
	LSB(USBD_VID),              /* idVendor */
	MSB(USBD_VID),              /* idVendor */
	LSB(USBD_PID),              /* idVendor */
	MSB(USBD_PID),              /* idVendor */
	0x00,                       /* bcdDevice rel. 2.00 */
	0x00,
	0x01,                       /* Index of manufacturer string */
	0x02,                       /* Index of product string */
	0x03,                       /* Index of serial number string */
	0x01                        /* bNumConfigurations */
};

uint8_t device_qulifier_desc[] = {
	0x0A,			                // bLength
	eUSB_DT_DEVICE_QUALIFIER,		// bDescriptorType
	LSB(USB_11_SPEC_VER), 			// bcdUSB				: USB 1.1 Spec
	MSB(USB_11_SPEC_VER),
	0x00,							// bDeviceClass			: Device Class
	0x00,							// bDeviceSubClass		: Device Subclass
	0x00,							// bDeviceProtocol		: Device Protocol
	USB_EP0_MAX_PKT_SZ,				// bMaxPacketSize0		: MaxPacketSize
	0x01,							// bNumConfigurations	: One Configuration
	0x00
};

uint8_t config_desc[] = {
    /****************************/
	/* Congifuration Descriptor */
	/****************************/
	eUSB_DESC_CONFIG_LEN,			// bLength				: 0x09
	eUSB_DT_CONFIGURATION,			// bDescriptorType		: CONFIGURATION
	0x00,							// wTotalLength			: assigned by process
	0x00,
	0x01,				            // bNumInterface		: 1
	0x01,							// bConfigurationValue	: 0x01 
	0x00,							// iConfiguration		: N/A 
	0xC0,							// bmAttribute			: Self-powered
	0xFA,							// iMaxPower 			: 
    
    /******************************************/
	/* HID Interface Descriptor (Interface 0) */
	/******************************************/
	// Standard HID Interface Descriptor
	eUSB_DESC_INTERFACE_LEN,	// bLength
	eUSB_DT_INTERFACE,			// bDescriptorType		: INTERFACE
	0x00,                       // bInterfaceNumber		: Interface ID 0
	0x00,						// bAlternateSetting	: Alternate 0
	0x01,						// bNumEndpoints		: One Endpoint 
	eUSB_CLASS_HID,				// bInterfaceClass		: HID
	0x00,						// bInterfaceSubclass	: Unused
	0x00,						// bInterfaceProtocol	: Unused
	0x00,						// iInterface			: Unused

	//HID Class-Specific Descriptor
	eUSB_DESC_HID_LEN,			// bLength
	eUSB_DT_HID,				// bDescriptorType		: HID DESCRIPTOR
	LSB(USB_HID_SPEC_VER),		// bcdHID				: HID Spec 1.0
	MSB(USB_HID_SPEC_VER),
	0x00,						// bCountryCode			: None 
	0x01,						// bNumDescriptors		: One Class Descriptor
	eUSB_DT_REPORT,				// bClassDescriptorType	: Report Descriptor
	0x23,						// HID descriptor size (LSB)
	0x00,						// HID descriptor size (MSB)

	// HID IN Endpoint Descriptor
	eUSB_DESC_EP_LEN,			// bLength
	eUSB_DT_ENDPOINT,			// bDescriptorType		: ENDPOINT
	USB_EP1_INTR|USB_IN,		// bEndpointAddress		: Endpoint Number 1 (Interrupt IN)
	eUSB_XferINTR,				// bmAttributes 		: Interrupt
	LSB(HID_INTR_MAX_PKT_SZ),	// wMaxPacketSize		: 1byte
	MSB(HID_INTR_MAX_PKT_SZ),
	HID_INTR_INTERVAL,			// bInterval			: 16ms
};

uint8_t AccHIDReportDesc[] = {	/* media playback remote HID report descriptor */
	HID_USAGE_PAGE, HID_CONSUMER_DEVICE,	// Usage Page (Consumer Devices)	/* Global Item */
		
	HID_1BYTE_USAGE, HID_COLLECTION_APP,			// Usage (Consumer Control) 	/* Local Item */
		
	HID_COLLECTION, 0x01,					// Collection (Application)
		
		HID_LOGICAL_MIN,	0x00,		// Logical Minimum (0)
		HID_LOGICAL_MAX,	0x01,		// Logical Maximum (1)
			
		HID_REPORT_SZ,		0x01,		// Report Size	(1)
		HID_REPORT_CNT, 	0x06,		// Report Count (6)
		
		HID_1BYTE_USAGE, 0xE9,			// Usage (Volume Up)
		HID_1BYTE_USAGE, 0xEA,			// Usage (Volume Down)
		HID_1BYTE_USAGE, 0xE2,			// Usage (Mute)
		HID_1BYTE_USAGE, 0xCD,			// Usage (Play/Pause)
		HID_1BYTE_USAGE, 0xB5,			// Usage (Scan Next Track)
		HID_1BYTE_USAGE, 0xB6,			// Usage (Scan Previous Track)
			
		HID_INPUT, 0x02,			// Input (Data, Var, Abs)
	
		HID_REPORT_SZ,		0x02,		// Report Size	(2)
		HID_REPORT_CNT, 	0x01,		// Report Count (1)
	
		HID_INPUT, 0x03,			// Input (Cnst, Var, Abs)
	
	HID_END_COLLECTION, 		// End Collection
};

uint8_t lang_desc[] = { //code
	0x04,				// bLength			: Size of this descriptor
	eUSB_DT_STRING,		// bDescriptorType	: String
	0x09, 0x04,			// bLang			: Language ID, 0409: English, 0404: Chinese Taiwan
};
