#ifndef _USB_H_
#define _USB_H_

#include "A31G31x.h"
//#include "def.h"
#include "aa_types.h"

#define USBD_VID 0x4444
#define USBD_PID 0x4444

#define MSB(word) 		(uint8_t)(((uint16_t)(word) >> 8) & 0xff)
#define LSB(word)		(uint8_t)((uint16_t)(word) & 0xff)

// USB Spec version
#define USB_11_SPEC_VER     (0x0110)

// HID Class 1.0 Spec version
#define USB_HID_SPEC_VER    (0x0100)

#define USB_EP0_MAX_PKT_SZ		0x40
#define eUSB_DESC_CONFIG_LEN    0x09
#define eUSB_DESC_INTERFACE_LEN 0x09
#define eUSB_DESC_HID_LEN       0x09
#define eUSB_DESC_EP_LEN        0x07

#define USB_EP0_CTRL		0
#define USB_EP1_INTR		1	/* for tx/rx interrupt */

/* USB standard class */
#define USB_IN			0x80
#define USB_IN_MASK		0x7F

#define HID_INTR_MAX_PKT_SZ	4
#define HID_INTR_INTERVAL	16

/* USB Request Type */
typedef enum{
	eUSB_REQ_TYPE_STANDARD	= 0x0,
	eUSB_REQ_TYPE_CLASS		= 0x1,
	eUSB_REQ_TYPE_VENDOR	= 0x2,
}eUSB_ReqType;

typedef enum{
	eUSB_REQ_RECIPIENT_DEVICE	=0x00,
	eUSB_REQ_RECIPIENT_INTERFACE,
	eUSB_REQ_RECIPIENT_ENDPOINT,
	eUSB_REQ_RECIPIENT_MASK
}eUSB_Request;

typedef enum{
	eUSB_REQ_GET_STATUS			=0x00,
	eUSB_REQ_CLEAR_FEATURE		=0x01,
	eUSB_REQ_SET_FEATURE		=0x03,
	eUSB_REQ_SET_ADDRESS		=0x05,
	eUSB_REQ_GET_DESCRIPTOR		=0x06,
	eUSB_REQ_SET_DESCRIPTOR		=0x07,
	eUSB_REQ_GET_CONFIGURATION	=0x08,
	eUSB_REQ_SET_CONFIGURATION	=0x09,
	eUSB_REQ_GET_INTERFACE		=0x0A,
	eUSB_REQ_SET_INTERFACE		=0x0B,
	eUSB_REQ_SYNCH_FRAME		=0x0C
}eUSB_StdDevReq;

typedef enum{
	eUSB_DT_DEVICE			= 0x01,
	eUSB_DT_CONFIGURATION	= 0x02,
	eUSB_DT_STRING			= 0x03,
	eUSB_DT_INTERFACE		= 0x04,
	eUSB_DT_ENDPOINT		= 0x05,
	eUSB_DT_DEVICE_QUALIFIER= 0x06,
	eUSB_DT_OTHER_SPEED_CONFIGURATION = 0x07,
	eUSB_DT_HID				= 0x21,
	eUSB_DT_REPORT			= 0x22,
	eUSB_DT_PHYSICAL		= 0x23,
	eUSB_DT_CS_INTERFACE	= 0x24,   // Audio Class-Specific Descriptor
	eUSB_DT_CS_ENDPOINT		= 0x25,   // Audio Class-Specific Descriptor
	eUSB_DT_DFU_FUNCTIONAL	= 0x21,
	eUSB_DT_DFU_BOS			= 0xFF,
}eUSB_DescriptorType;

typedef enum{
	eUSB_XferCTRL	= 0x00,
	eUSB_XferISO	= 0x01,
	eUSB_XferBULK	= 0x02,
	eUSB_XferINTR	= 0x03,
}eUSB_XferType;

typedef enum{
	eUSB_CLASS_HID		= 0x03,
}eUSB_CLASS_CODE;

typedef struct{
    uint8_t bmRequestType;
    uint8_t bRequest;
    uint16_t wValue;
    uint16_t wIndex;
    uint16_t wLength;
}tSetup_stage_data_packet;


/* HID class */

#define HID_CONSUMER_DEVICE		0x0C
#define HID_USAGE_PAGE			0x05	/* 1byte size */
#define HID_1BYTE_USAGE         0x09
#define HID_2BYTE_USAGE         0x0A
#define HID_COLLECTION			0xA1
#define HID_END_COLLECTION		0xC0
#define HID_COLLECTION_APP		0x01
#define HID_COLLECTION_LOGICAL	0x02
#define HID_LOGICAL_MIN			0x15	/* 1byte size */
#define HID_LOGICAL_MAX			0x25	/* 1byte size */
#define HID_REPORT_SZ			0x75	/* 1byte size */
#define HID_REPORT_CNT			0x95	/* 1byte size */
#define HID_INPUT				0x81	/* 1byte size */
#define HID_1BYTE_USAGE_MINIMUM 0x19
#define HID_1BYTE_USAGE_MAXIMUM 0x29
#define HID_2BYTE_USAGE_MINIMUM 0x1A
#define HID_2BYTE_USAGE_MAXIMUM 0x2A

typedef enum{
	eHIDCMD_Stop		= 0,
	eHIDCMD_VolUp,
	eHIDCMD_VolDown,
	eHIDCMD_Mute,
	eHIDCMD_PlayPause,
	eHIDCMD_ScanNext,
	eHIDCMD_ScanPrev,
}eHID_cmd;


/* Others */
// define
#define SWAP16(s) (((((s) & 0xff) << 8) | (((s) >> 8) & 0xff)))
#define SWAP32(l) (((((l) & 0xff000000) >> 24) | (((l) & 0x00ff0000) >> 8)  | (((l) & 0x0000ff00) << 8)  | (((l) & 0x000000ff) << 24)))

// Variables 
extern uint8_t device_desc[];
extern uint8_t device_qulifier_desc[];
extern uint8_t config_desc[];
extern uint8_t AccHIDReportDesc[];
extern uint8_t lang_desc[];
// Functions
void USB_Init(void);
void USB_Send_HID_Report_EP1(uint32_t opcode);
#endif
