#include "A31G31x_pcu.h"
#include "A31G31x_scu.h"
#include "usb.h"


uint8_t device_desc[] = {
	0x12,                       // bLength
	eUSB_DT_DEVICE,             // bDescriptorType		: DEVICE
	LSB(USB_20_SPEC_VER),       // bcdUSB				: USB 2.0 Spec
	MSB(USB_20_SPEC_VER),       
	0x02,                       // bDeviceClass			: Device Class
	0x00,                       // bDeviceSubClass		: Device Subclass
	0x00,                       // bDeviceProtocol		: Device Protocol
	USB_EP0_MAX_PKT_SZ,         // bMaxPacketSize0		: MaxPacketSize
	LSB(USBD_VID),              // idVendor				: Vendor ID
	MSB(USBD_VID),              
	LSB(USBD_PID),              // idProduct			: Product ID
	MSB(USBD_PID),              
	0x00,                       // bcdDevice			: Device Release Number
	0x02,                       
	0x01,                       // iManufacturer		: Manufacturer String Index
	0x02,                       // iProduct				: Product String Index
	0x03,                       // iSerialNumber		: Serial Number String Index
	0x01                        // bNumConfigurations	: One Configuration
};

uint8_t device_qulifier_desc[] = {
	0x0A,			                // bLength
	eUSB_DT_DEVICE_QUALIFIER,		// bDescriptorType
	LSB(USB_20_SPEC_VER), 			// bcdUSB				: USB 2.0 Spec
	MSB(USB_20_SPEC_VER),
	0x00,							// bDeviceClass			: Device Class
	0x00,							// bDeviceSubClass		: Device Subclass
	0x00,							// bDeviceProtocol		: Device Protocol
	USB_EP0_MAX_PKT_SZ,				// bMaxPacketSize0		: MaxPacketSize
	0x01,							// bNumConfigurations	: One Configuration
	0x00
};

uint8_t config_desc[] = {
    /****************************/
	/* Congifuration Descriptor */
	/****************************/
	eUSB_DESC_CONFIG_LEN,			// bLength				: 0x09
	eUSB_DT_CONFIGURATION,			// bDescriptorType		: CONFIGURATION
	0x00,							// wTotalLength			: assigned by process
	0x00,
	0x02,				            // bNumInterface		: 1
	0x01,							// bConfigurationValue	: 0x01 
	0x00,							// iConfiguration		: N/A 
	0xC0,							// bmAttribute			: Self-powered
	0x32,							// iMaxPower 			: 
    
    /**************************************************/
	/* CDC Command Interface Descriptor (Interface 0) */
	/**************************************************/
	// Standard CDC Interface Descriptor
	eUSB_DESC_INTERFACE_LEN,	// bLength
	eUSB_DT_INTERFACE,			// bDescriptorType		: INTERFACE
	0x00,                       // bInterfaceNumber		: Interface ID 0
	0x00,						// bAlternateSetting	: Alternate 0
	0x01,						// bNumEndpoints		: One Endpoint 
	eUSB_CLASS_CDC,				// bInterfaceClass		: CDC
	0x02,						// bInterfaceSubclass	: Abstract Control Model
	0x01,						// bInterfaceProtocol	: Common AT commands
	0x00,						// iInterface			: Unused

	// Header Functional Descriptor
	0x05,   					// bLength				: Endpoint Descriptor size
	0x24,   					// bDescriptorType		: CS_INTERFACE
	0x00,   					// bDescriptorSubtype	: Header Func Desc
	0x10,   					// bcdCDC				: spec release number
	0x01,
	
	// Call Management Functional Descriptor
	0x05,						// bFunctionLength
	0x24,						// bDescriptorType		: CS_INTERFACE
	0x01,						// bDescriptorSubtype	: Call Management Func Desc
	0x00,						// bmCapabilities		: D0+D1
	0x01,						// bDataInterface		: 1

	// ACM Functional Descriptor
	0x04,						// bFunctionLength
	0x24,						// bDescriptorType		: CS_INTERFACE
	0x02,						// bDescriptorSubtype	: Abstract Control Management desc
	0x02,						// bmCapabilities

	// Union Functional Descriptor
	0x05,						// bFunctionLength */
	0x24,						// bDescriptorType		: CS_INTERFACE */
	0x06,						// bDescriptorSubtype	: Union func desc */
	0x00,						// bMasterInterface		: Communication class interface */
	0x01,						// bSlaveInterface0		: Data Class Interface */

	// Endpoint 2 Descriptor
	0x07,						// bLength: Endpoint Descriptor size
	eUSB_DT_ENDPOINT,			// bDescriptorType		: Endpoint
	USB_EP2_CDC_CMD,			// bEndpointAddress
	0x03,						// bmAttributes			: Interrupt
	LSB(CDC_CMD_PACKET_SIZE),	// wMaxPacketSize		:
	MSB(CDC_CMD_PACKET_SIZE),
	0x10,                           /* bInterval: */ 
	
    /**************************************************/
	/* CDC IN/OUT Interface Descriptor (Interface 1) */
	/**************************************************/
	// Data class interface descriptor
	0x09,   					// bLength: Interface Descriptor size
	eUSB_DT_INTERFACE,    		// bDescriptorType		:
	0x01,   					// bInterfaceNumber		: Number of Interface
	0x00,   					// bAlternateSetting	: Alternate setting
	0x02,   					// bNumEndpoints		: Two endpoints used
	0x0A,   					// bInterfaceClass		: CDC
	0x00,   					// bInterfaceSubClass	:
	0x00,						// bInterfaceProtocol	:
	0x00,						// iInterface			:
	
	// Endpoint OUT Descriptor
	0x07,						// bLength				: Endpoint Descriptor size
	eUSB_DT_ENDPOINT,			// bDescriptorType		: Endpoint
	USB_EP1_CDC_OUT,			// bEndpointAddress
	0x02,						// bmAttributes			: Bulk
	LSB(CDC_DATA_FS_MAX_PACKET_SIZE),	// wMaxPacketSize:
	MSB(CDC_DATA_FS_MAX_PACKET_SIZE),
	0x00,						// bInterval			: ignore for Bulk transfer

	// Endpoint IN Descriptor
	0x07,   /* bLength: Endpoint Descriptor size */
	eUSB_DT_ENDPOINT,      /* bDescriptorType: Endpoint */
	USB_EP1_CDC_IN,                         /* bEndpointAddress */
	0x02,                              /* bmAttributes: Bulk */
	LSB(CDC_DATA_FS_MAX_PACKET_SIZE),  /* wMaxPacketSize: */
	MSB(CDC_DATA_FS_MAX_PACKET_SIZE),
	0x00  
};

uint8_t lang_desc[] = { //code
	0x04,				// bLength			: Size of this descriptor
	eUSB_DT_STRING,		// bDescriptorType	: String
	0x09, 0x04,			// bLang			: Language ID, 0409: English, 0404: Chinese Taiwan
};




uint8_t _cdc_line_coding_def[] ={
	((uint32_t)BAUD_RATE)>>0 & 0xFF,
	((uint32_t)BAUD_RATE)>>8 & 0xFF,
	((uint32_t)BAUD_RATE)>>16 & 0xFF,
	((uint32_t)BAUD_RATE)>>24 & 0xFF,
	(uint8_t)CTOP_BIT,
	(uint8_t)PARITY,
	(uint8_t)NUM_OF_BIT
};
