#include "usb.h"
#include "stdio.h"
#include "A31G31x.h"

uint32_t   USB_DOEP0_DMA_DESC_RX[2];
uint32_t   USB_DOEP0_DMA_BUF_RX[64];
uint32_t   USB_DIEP0_DMA_DESC_TX[2];
uint32_t   USB_DIEP0_DMA_BUF_TX[64];

uint32_t   USB_DOEP1_DMA_DESC_RX[2];
uint32_t   USB_DOEP1_DMA_BUF_RX[64];
uint32_t   USB_DIEP1_DMA_DESC_TX[2];
uint32_t   USB_DIEP1_DMA_BUF_TX[64];

uint32_t   USB_DIEP2_DMA_DESC_TX[2];
uint32_t   USB_DIEP2_DMA_BUF_TX[64];

uint32_t   USB_EP0_status = 0;
uint32_t   USB_EP1_status = 0;


tSetup_stage_data_packet _setup_data_packet;
tCdc_line_coding _cdc_line_coding;

#define SOFCTMR_TEST
#ifdef SOFCTMR_TEST

#endif

void USB_ReadyTx(uint16_t length, uint8_t *desc){
	USB_DIEP0_DMA_DESC_TX[0] = 0x0A000000 | length;
    USB_DIEP0_DMA_DESC_TX[1] = (uint32_t)desc;
    
	USB->GRSTCTL = 0x80000020;   // TX FIFO FLUSH
    
	while(USB->GRSTCTL & (0x1 << 5)) { __nop(); }
        
    USB->DIEPDMA0 = (uint32_t)USB_DIEP0_DMA_DESC_TX;
    USB->DIEPCTL0 = 0
		| (0x0   <<  0) //64Byte
		| (0x1UL << 31)  //EPEna
		;
}	

void USB_ReadyRx(uint16_t length, uint8_t *desc){
	USB_DOEP0_DMA_DESC_RX[0] = 0x0A000000 | length;// | SWAP16(_setup_data_packet.wLength);
	USB_DOEP0_DMA_DESC_RX[1] = (uint32_t)desc;
	
	
	USB->DOEPDMA0 = (uint32_t)USB_DOEP0_DMA_DESC_RX;
	USB->DOEPCTL0 = 0
		| (0x0   <<  0) //64Byte
		| (0x1UL << 31) //EPEna
		;
}

void USB_DataStageIN(uint16_t length, uint8_t *desc){
	USB_EP0_status = eUSB_EP0_DATA_IN;
	USB_ReadyTx(length, desc);
}

void USB_DataStageOUT(uint16_t length, uint8_t *desc){
	USB_EP0_status = eUSB_EP0_DATA_OUT;
	USB_ReadyRx(length, desc);
}

void USB_prepareSetupPacket(){
	USB_EP0_status = eUSB_EP0_SETUP;
	USB_ReadyRx(0x08, (uint8_t*)&_setup_data_packet);

}
void USB_StatusStageOut(){
	USB_EP0_status = eUSB_EP0_STATUS_OUT;
	USB_prepareSetupPacket();
}

void USB_StatusStageIn(){
	uint8_t tmp_zero = 0;
	USB_ReadyTx(0, &tmp_zero);
}


void USB_Send_DATA_EP1(uint8_t ch){
	uint32_t timeout;
	USB_DIEP1_DMA_DESC_TX[0] = 0x0A000000 | 0x1;	//1Byte
	USB_DIEP1_DMA_DESC_TX[1] = (uint32_t)&ch;
			
	USB->DIEPDMA1 = (uint32_t)USB_DIEP1_DMA_DESC_TX;
		
	USB->DIEPCTL1 |= (0x1   <<  0); //1Byte
	USB->DIEPCTL1 |= (0x1   << 26); //CNAK
	USB->DIEPCTL1 |= (0x1UL << 31); //EPEna
	timeout = 10000;
	while(!(((USB_DIEP1_DMA_DESC_TX[0] >> 30) & 0x3) == 0x3
		|| ((USB_DIEP1_DMA_DESC_TX[0] >> 30) & 0x3) == 0x2)){if(!(timeout--)) break;}
}

void USB_Receive_DATA_EP1(uint8_t *ch){
	USB_DOEP1_DMA_DESC_RX[0] = 0x0A000000 | 0x1;	//1Byte
	USB_DOEP1_DMA_DESC_RX[1] = (uint32_t)ch;
		
	USB->DOEPDMA1 = (uint32_t)USB_DOEP1_DMA_DESC_RX;
	
	USB->DOEPCTL1 |= (0x64  <<  0); //1Byte
	USB->DOEPCTL1 |= (0x1   << 26); //CNAK
	USB->DOEPCTL1 |= (0x1UL << 31); //EPEna
}


void USB_GINTSTS_SOF(){  //[3]
	#ifdef SOFCTMR_TEST

	#endif
}


void USB_GINTSTS_USBRst(){  //[12]
    USB->DOEPCTL0 = (1 << 27)    ;
    USB->DOEPCTL1 = (1 << 27)    ;
    USB->DOEPCTL2 = (1 << 27)    ;
    USB->DOEPCTL3 = (1 << 27)    ;
    USB->DOEPCTL4 = (1 << 27)    ;
    USB->DIEPCTL0 = (1 << 27)    ;
    USB->DIEPCTL1 = (1 << 27)    ;
    USB->DIEPCTL2 = (1 << 27)    ;
    USB->DIEPCTL3 = (1 << 27)    ;
    USB->DIEPCTL4 = (1 << 27)    ;

	
    USB->DAINTMSK = 0
		| ( 1 << 16)	//OUTEP0
		| ( 1 << 0 )	//INEP0
		;
	
    USB->DOEPMSK = 0
		| ( 1 << 3 )	//SetUPMsk
		| ( 1 << 0 )	//XferComplMsk
		;
    USB->DIEPMSK = 0
		| ( 1 << 3 )	//TimeOUTMsk
		| ( 1 << 0 )	//XferComplMsk
		;
		
    USB->GINTMSK = 0
		| (0x1 << 19)	//DOEPINT EN
		| (0x1 << 18)	//DIEPINT EN
		;
	
	USB_CoreInit();
	
	//FIFO size configuration
    USB->GRXFSIZ 	= 0x00000080;
	USB->GNPTXFSIZ 	= 0x00400080;
	USB->DIEPTXF1 	= 0x004000C0;
    USB->DIEPTXF2 	= 0x00400100;
    
}


void USB_GINTSTS_EnumDone(){
    // Initialization on Enumeration Completion
    USB->DIEPCTL0 = (0x0 << 0);      //64 Byte
    USB->DOEPCTL0 = (0x1UL << 31);   //EPEna
	
    USB->GINTMSK  |= (0x1<<3 );		//SofMsk Unmask
	
	//Wait Setuppacket
    USB_prepareSetupPacket();
}


void USB_GetDescriptor(){
	uint16_t req, len;
	uint8_t *desc;
	
	req = SWAP16(_setup_data_packet.wValue)&0xFF;
	switch(req){
		case eUSB_DT_DEVICE:
			desc = device_desc;
			len = device_desc[0];
			break;
		case eUSB_DT_CONFIGURATION:
			desc = config_desc;
			len = _setup_data_packet.wLength & 0xffff;
			if(len == 0xff){
				len = 0x09;
			}
			break;
		case eUSB_DT_STRING:
			desc = lang_desc;
			len = lang_desc[0];
			break;
		case eUSB_DT_DEVICE_QUALIFIER:
			desc = device_qulifier_desc;
			len = device_qulifier_desc[0];
			break;
		case eUSB_DT_OTHER_SPEED_CONFIGURATION:
			break;
		default:
			//control error
			return;
	}
	USB_DataStageIN(len, desc);
}


void USB_SetAddress(){
	uint16_t addr;
	
	addr = _setup_data_packet.wValue & 0x7f;
	USB->DCFG &= ~(0x7f << 4); //clear
	USB->DCFG |= ((addr&0x7f) << 4);
	USB_StatusStageIn();
}
uint8_t test_ch =0;


void USB_SetConfiguration(){
	//CDC Data
	USB->DIEPCTL1 |= 0
		| (0x1 << 22)	// Tx FIFO Num
		| (0x2 << 18)	// Endpoint Type : BULK
		| (0x1 << 15)	// Endpoint is Activated
		;
    USB->DOEPCTL1 |= 0
		| (0x2 << 18)	// Endpoint Type : BULK
		| (0x1 << 15)	// Endpoint is Activated
		;
    USB->DAINTMSK |= 0
		|(1 << 17)	//OUTEP1
		|(1 << 1 )	//INEP1
		;
	
	//CDC Command
	USB->DIEPCTL2 |= 0
		| (0x1 << 22)	// Tx FIFO Num
		| (0x3 << 18)	// Endpoint Type : Interrupt
		| (0x1 << 15)	// Endpoint is Activated
		;
	
    USB->DAINTMSK |= 0
		|(1 << 2 )	//INEP2
		;
	USB_StatusStageIn();
	USB_Receive_DATA_EP1(&test_ch);
}


void USB_Standard_Device_Request(){
	uint16_t req;
	req = _setup_data_packet.bRequest;
	switch(req){
		case eUSB_REQ_GET_DESCRIPTOR: 
			USB_GetDescriptor();
        break;
    
		case eUSB_REQ_SET_ADDRESS:
			USB_SetAddress();
		break;
    
		case eUSB_REQ_SET_CONFIGURATION:
			USB_SetConfiguration();
		break;
    
		case eUSB_REQ_GET_CONFIGURATION:                 
		break;
    
		case eUSB_REQ_GET_STATUS:                                  
		break;

		case eUSB_REQ_SET_FEATURE:   
		break;
    
		case eUSB_REQ_CLEAR_FEATURE:                                   
		break;
		
		default:  
		//Controll Error
		break;
  }
}

void USB_Standard_Interface_Request(){
	uint16_t reqType, req, val, len;
	reqType = (_setup_data_packet.bmRequestType>>5) & 0x3;
	req = _setup_data_packet.bRequest;
	val = SWAP16(_setup_data_packet.wValue);
	len = _setup_data_packet.wLength;
	
	switch(reqType){
		case eUSB_REQ_TYPE_STANDARD:
			switch(req){
				case eUSB_REQ_GET_DESCRIPTOR:
					switch(val){
						case eUSB_DT_REPORT:
							__nop();
							break;
						default:
							break;
					}
					break;
				case eUSB_REQ_GET_INTERFACE:
					USB_StatusStageIn();
					break;
				default:
					break;
			}
			break;
			
		case eUSB_REQ_TYPE_CLASS:
			switch(req){
				case eUSB_CDC_SEND_ENCAPSULATED_COMMAND:
					break;
				case eUSB_CDC_GET_ENCAPSULATED_RESPONSE:
					break;
				case eUSB_CDC_SET_COMM_FEATURE:
					break;
				case eUSB_CDC_GET_COMM_FEATURE:
					break;
				case eUSB_CDC_CLEAR_COMM_FEATURE:
					break;
				case eUSB_CDC_SET_LINE_CODING:
					USB_DataStageOUT(len, (uint8_t*) &_cdc_line_coding);
					break;
				case eUSB_CDC_GET_LINE_CODING:
					USB_DataStageIN(len, _cdc_line_coding_def);
					break;
				case eUSB_CDC_SET_CONTROL_LINE_STATE:
					break;
				case eUSB_CDC_SEND_BREAK:
					break;
				default:
					break;
			}
			
			
			break;
	}
}


void USB_Standard_EndPoint_Request(){
	__nop();
}


void USB_SetupStage(){
	uint16_t req;
	req = _setup_data_packet.bmRequestType;
	USB_EP0_status = eUSB_EP0_SETUP;
	switch(req&0x1F){
		case eUSB_REQ_RECIPIENT_DEVICE:
			USB_Standard_Device_Request();
			break;
		case eUSB_REQ_RECIPIENT_INTERFACE:
			USB_Standard_Interface_Request();
			break;
		case eUSB_REQ_RECIPIENT_ENDPOINT:
			USB_Standard_EndPoint_Request();
			break;
		default:
			//Stall
			break;
		
	}
}

void USB_OEPInt_DAINT_OutEPInt0_Combination(){
    unsigned int    usb_outepint0;
    unsigned int    DESC;
    unsigned int    IOC, SPD, SI;
    IOC = 0;
    SPD = 0;
	SI = 0;
	
    //IN Setup Stage
    usb_outepint0 = USB->DOEPINT0;

    IOC = (usb_outepint0 >> 0) & 0x1;
    SPD = (usb_outepint0 >> 3) & 0x1;
	SI = (usb_outepint0 >> 5) & 0x1;

    //CASE A
    if(SI == 0 && SPD == 0 && IOC == 1){
		USB->DOEPINT0 = (0x1 << 0);
        DESC = USB_DOEP0_DMA_DESC_RX[0];

		//SR
		if(((DESC >> 24) & 0x01) == 1){
			USB_SetupStage();
        }else{
			if(USB_EP0_status == eUSB_EP0_SETUP){
				//Status IN Stage
				USB_StatusStageIn();
				USB_prepareSetupPacket();
			}else{
				//Data OUT Stage
				USB_StatusStageIn();
				USB_prepareSetupPacket();
			}
		}
				
    //CASE B
    }else if(SI == 0 && SPD == 1 && IOC == 0){
	    USB->DOEPINT0 = (0x1 << 3);
		
		if(USB_EP0_status == eUSB_EP0_DATA_IN){
			// Data IN & Status OUT Stage
			USB->DIEPCTL0 |= (0x1<<26); //CNAK
			USB->DOEPCTL0 |= (0x1<<26); //CNAK
			USB_prepareSetupPacket();
			
		}else if(USB_EP0_status == eUSB_EP0_DATA_OUT){
			// Data Out Stage
			USB->DIEPCTL0 |= (0x1<<26); //CNAK
			USB->DOEPCTL0 |= (0x1<<26); //CNAK

			USB->DOEPCTL0 = 0
				| (0x0   <<  0) //64Byte
				| (0x1UL << 31) //EPEna
				;
			USB_EP0_status = eUSB_EP0_STATUS_IN;
			
		}else{
			// L : last descriptor
			DESC = USB_DOEP0_DMA_DESC_RX[0];

			USB->DIEPCTL0 |= (0x1<<26); //CNAK
			USB->DOEPCTL0 |= (0x1<<26); //CNAK

			if((DESC >> 27) & 0x1){
				// Status IN Stage
				USB_StatusStageIn(); 
				USB_prepareSetupPacket();
			}else{
				// Data Out Stage
				USB->DOEPCTL0 = 0
					| (0x0   <<  0) //64Byte
					| (0x1UL << 31) //EPEna
					;
			}
		}
		
    //CASE C
    } else if(SI == 0 && SPD == 1 && IOC == 1){
		//printf("case C\n\r");
        USB->DOEPINT0 = (0x1 << 3) | (0x1 << 0);
		
        DESC = USB_DOEP0_DMA_DESC_RX[0];
        //Prepare IN DATA and Status Stage
        
		//SR
		if(((DESC >> 24) & 0x01) == 1){
			USB_SetupStage();
        }
		USB->DOEPCTL0 |= (0x1<<26); //CNAK
		USB->DIEPCTL0 |= (0x1<<26); //CNAK
		
	//CASE D
    } else if(SI == 1 && SPD == 0 && IOC == 0){
		USB->DOEPINT0 = (0x1 << 5);
		USB->DIEPCTL0 |= (0x1<<26); //CNAK
		
	//CASE E
    } else if(SI == 1 && SPD == 0 && IOC == 1){
		DESC = USB_DOEP0_DMA_DESC_RX[0];
		//Data IN Stage 
		USB->DOEPINT0 = (0x1 << 5) | (0x01 << 0);
		USB->DIEPCTL0 |= (0x1<<26); //CNAK
		
		if((DESC >> 27) & 0x1){
			USB_StatusStageIn(); 
			USB_prepareSetupPacket();
		}
	}
}

void USB_OEPInt_DAINT_OutEPInt0(){
    uint32_t usb_outepint0;
    usb_outepint0 = USB->DOEPINT0;

	if (((usb_outepint0 & (1 <<  3))) || (usb_outepint0 & (1 <<  0))){
        USB_OEPInt_DAINT_OutEPInt0_Combination();
    }

}
void USB_OEPInt_DAINT_OutEPInt1(){
    uint32_t usb_outepint1;
    usb_outepint1 = USB->DOEPINT1;

	if (usb_outepint1 & (1 << 0)){ // XferCompl
        USB->DOEPINT1 = (1 << 0);
		USB_Send_DATA_EP1(test_ch);
		USB_Receive_DATA_EP1(&test_ch);
    }

}

void USB_IEPInt_DAINT_InEPInt0(){
    uint32_t usb_inepint0;

    usb_inepint0 = USB->DIEPINT0;

    //XferCompl
    if(usb_inepint0 & (1 << 0 )){ 
        USB->DIEPINT0 = (0x1 << 0);
		if(USB_EP0_status == eUSB_EP0_STATUS_IN){
			USB_prepareSetupPacket();
		}
		 
    }
}

void USB_IEPInt_DAINT_InEPInt1(){
	uint32_t usb_inepint1;
	usb_inepint1 = USB->DIEPINT1;

    //XferCompl
	if(usb_inepint1 & (1<<0)){
		USB->DIEPINT1 = (0x1 << 0);
	}
}

void USB_GINTSTS_OEPInt(){
	uint32_t usb_daint;
    usb_daint = USB->DAINT;
	//[OEPInt DAINT]

    //OutEPInt0
    if(usb_daint & (1 << 16)){
        USB_OEPInt_DAINT_OutEPInt0();
    }
	//OutEPInt1
    if(usb_daint & (1 << 17)){
		USB_OEPInt_DAINT_OutEPInt1();
    }
}

void USB_GINTSTS_IEPInt(){
    uint32_t usb_daint;
    
	//[IEPInt DAINT]
	usb_daint = USB->DAINT;

    //INEPInt0
    if(usb_daint & (1 <<  0)){
        USB_IEPInt_DAINT_InEPInt0();
    }
	//INEPInt1
    if(usb_daint & (1 <<  1)){
        USB_IEPInt_DAINT_InEPInt1();
    }
}

void USB_GINTSTS_Suspend(){
	
	//USB->PCGCCTL |= (1<<0);	// Stop Phy Clock
	
	USB->DCTL = (1<<1);
	USB->GRSTCTL = 0x80000001;
	while(USB->GRSTCTL & (0x1 << 0)) { __nop(); }
	
	//USB->PCGCCTL &= ~(1<<0);	// Stop Phy Clock
	USB_CoreInit();
}

void USB_IRQHandler(){

	uint32_t usb_global_int_sts;
	usb_global_int_sts = USB->GINTSTS;
		
	//OEPInt
	if(usb_global_int_sts & (1 << 19)){ 
		USB_GINTSTS_OEPInt();
	}
	
	//IEPInt
	if(usb_global_int_sts & (1 << 18)){ 
		USB_GINTSTS_IEPInt();
	}
	
	//EnumDone
	if(usb_global_int_sts & (1 << 13)){ 
		USB_GINTSTS_EnumDone();
		USB->GINTSTS = (1 << 13);
	}
	
	//USBRst
	if(usb_global_int_sts & (1 << 12)){ 
		USB_GINTSTS_USBRst();
		USB->GINTSTS = (1 << 12);
	}
	
	//USBSusp
	if(usb_global_int_sts & (1 << 11)){ 
		USB_GINTSTS_Suspend();
		USB->GINTSTS = (1 << 11);
	}
	
	//ErlySusp
	if(usb_global_int_sts & (1 << 10)){ 
		USB->GINTSTS = (1 << 10);
	}
	
	//Sof
	if(usb_global_int_sts & (1 <<  3)){ 
		USB_GINTSTS_SOF();
		USB->GINTSTS = (1 << 3);
	}
}


void USB_CoreInit(){
	uint16_t len;
	
	USB->GUSBCFG = 0x0008A440;
	
    USB->DCFG = 0
		| (0x3 << 0)		//Device Speed
		;
	
    USB->DCTL = 0;
	
    USB->GAHBCFG = 0
		| (0x1 <<5 )	//DMA Mode 
		| (0x5 <<1 )	//Burst length 8
		| (0x1 <<0 )	//glblIntrMsk
		;
		
	USB->DTHRCTL = 0x0;

    USB->DCTL = 0;

	USB->GINTSTS  = 0x54000024;

    USB->DCFG     = 0x0A841003;

    USB->DCTL     = 0x00020000;

    USB->GINTMSK  |= 0
		| (0x1<<12)		//USBRstMsk Unmask
		| (0x1<<13)		//EnumDoneMsk Unmask
		| (0x1<<10)		//ErlySuspMsk Unmask
		| (0x1<<11)		//USBSuspMsk Unmask
		;
		
	//Insert Total Config Desc
	len = 67;

	config_desc[2] = LSB(len);
	config_desc[3] = MSB(len);
	
	#ifdef SOFCTMR_TEST

	#endif
}
void USB_Init(){

	
	SCU->PCER2 &=  ~(1<<16);
    SCU->PER2 &=  ~(1<<16);
	
	SCU->PCER2 |=  (1<<16);
    SCU->PER2 |=  (1<<16);

	NVIC_EnableIRQ(USB_IRQn);
	

//	USB->PCGCCTL |= (1<<0);	// Stop Phy Clock
//	
//	//Disconnect
//	USB->DCTL = (1<<1);
//	
//	//Reset
//	USB->GRSTCTL = 0x80000001;
//	while(USB->GRSTCTL & (0x1 << 0)) { __nop(); }
//	
//	USB->PCGCCTL &= ~(1<<0);	// Start Phy Clock
	
	USB_CoreInit();
}
